@@ -11,11 +11,11 @@ BEGIN {
 	use vars qw($VERSION $DEBUG @ISA @EXPORT @EXPORT_OK);
 
 	# set the version for version checking
-	$VERSION     = 1.10;
+	$VERSION     = 1.20;
 	@ISA         = qw(Exporter);
 	@EXPORT      = qw(&SetArch &SetDistArch &GetArch &GetDistArch
 			&GetAllowedArch &GetAllowedDistArch &TestArch &IsElement
-			&SetLocalRPMs &AddLocalRPMs &CheckLocal &SelectRPMs
+			&SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
 			&SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase);
 	@EXPORT_OK   = qw();
 }
@@ -31,6 +31,7 @@ my $TieRpmDatabase = 0;
 my $Arch;
 my $DistArch;
 my %LocalRPMs;
+my %LocalRPMsArchs;
 my @LocalTied;
 my %RPM;
 
@@ -119,7 +120,7 @@ sub getheader {
 		$hdr= $RPM{$self->name};
 		unless ($hdr)
 		{
-       		print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
+       			print STDERR "Error: " . $self->rpmname . ": $RPM::err\n";
 			$self->{TYPE} = "r";
 			return undef;
 		} else {
@@ -186,7 +187,6 @@ sub compare {
 	my $b = $rpmb->{NAME};
 	if ($a ne $b) { return ($a cmp $b); }
 	if ($rpma->version eq $rpmb->version) { return 0; }
-
 	if ($autoupdate::QueryRpmHeaders and $rpma and $rpmb) {
 		$a = $rpma->getheader();
 		$b = $rpmb->getheader();
@@ -229,7 +229,7 @@ sub compare {
 
 	# Split off equal parts first
 	my ($a1,$b1);
-	while ($a and $b) {
+	while ($a ne "" and $b ne "") {
 		($a1,$b1)= ("","");
 		if ($a =~ /^(\d+)(.*)$/) {
 			$a = $2;
@@ -432,7 +432,9 @@ sub GetAllowedDistArch {
 	# Safe default
 	my @allowedarch= ($DistArch, "noarch");
 
-	if ( IsElement($DistArch,qw(i386 i486 i586 i686 athlon)) ) {
+	if ($DistArch eq "src") {
+		@allowedarch=qw(src);
+	} elsif ( IsElement($DistArch,qw(i386 i486 i586 i686 athlon)) ) {
 		@allowedarch=qw(athlon i686 i586 i386 noarch);
 	} elsif ( IsElement($DistArch,qw(sparc sparc64)) ) {
 		@allowedarch=qw(sparc64 sparc noarch);
@@ -475,6 +477,7 @@ sub IsElement
 sub SetLocalRPMs
 {
 	%LocalRPMs= ();
+	%LocalRPMsArchs= ();
 	SetQueryRpmDatabase(0);
 	AddLocalRPMs( @_ ) if (@_);
 }
@@ -485,20 +488,27 @@ sub SetLocalRPMs
 
 sub AddLocalRPMs
 {
-	my ($item, $tmp);
+	my ($item, $tmp, $name, $res, $loc);
 	my $type = "";
 	if ($_[0] =~ /^[flr]$/) {
 		$type= shift;
 	}
 	foreach $item ( @_ ) {
 		$tmp = Header->new($item,$type);
+		$name = $tmp->name;
 		next unless $tmp;
-		if ( CheckLocal($tmp->name) ) {
-			if ( $tmp->compare($LocalRPMs{$tmp->name})==1 ) {
-				$LocalRPMs{$tmp->name}= $tmp;
+		if ( CheckLocal($name) ) {
+			$loc= $LocalRPMs{$name};
+			$res= $tmp->compare($loc);
+			if ($res==1) {
+				$LocalRPMs{$name}= $tmp;
+				$LocalRPMsArchs{$name}= [ $tmp->arch ];
+			} elsif ($res ==0 and ($tmp->version eq $loc->version)) {
+				push(@{$LocalRPMsArchs{$name}}, $tmp->arch); 
 			}
 		} else {
-			$LocalRPMs{$tmp->name}=$tmp;
+			$LocalRPMs{$name}=$tmp;
+			$LocalRPMsArchs{$name}= [ $tmp->arch ];
 		}
 	}
 }
@@ -509,16 +519,18 @@ sub AddLocalRPMs
 
 sub CheckLocal
 {
-	if ( $LocalRPMs{$_[0]} ) {
-		return $LocalRPMs{$_[0]}->version;
+	my $name= shift;
+	my $hdr= $LocalRPMs{$name};
+	if ( $hdr ) {
+		return $hdr->version;
 	} elsif ($QueryRpmDatabase) {
 		TieRpmDatabase(1);
-		my $hdr= $RPM{$_[0]};
+		$hdr= $RPM{$name};
 		if ($hdr) {
 			my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
 			$tmp->{EPOCH}= $hdr->{EPOCH};
 			$tmp->{HDR}= $hdr;
-			$LocalRPMs{$_[0]}= $tmp;
+			$LocalRPMs{$name}= $tmp;
 			push(@LocalTied,$tmp);
 			return $tmp->version;
 		}
@@ -526,6 +538,30 @@ sub CheckLocal
 	return 0;
 }
 
+######################
+# Compare an rpm against the local version
+######################
+
+sub CompareLocal
+{
+	my $name= $_[0]->name;
+	my $hdr= $LocalRPMs{$name};
+	unless ( $hdr or !$QueryRpmDatabase) {
+		TieRpmDatabase(1);
+		$hdr= $RPM{$name};
+                if ($hdr) {
+                        my $tmp= Header->new($hdr->{NAME} . "-" . $hdr->{VERSION} . "-" . $hdr->{RELEASE} . "." . $hdr->{ARCH} . ".rpm","l");
+                        $tmp->{EPOCH}= $hdr->{EPOCH};
+                        $tmp->{HDR}= $hdr;
+                        $LocalRPMs{$name}= $tmp;
+                        push(@LocalTied,$tmp);
+                        $hdr=$tmp;
+                }
+	}
+	return 1 unless ($hdr);
+	return $_[0]->compare($hdr);
+}
+
 ######################
 # Should we try to query rpm headers
 ######################
@@ -602,7 +638,7 @@ sub SelectRPMs
 	my @rpms =();
 	my %Version=();
 	my %Test=();
-	my ($item, $rpm, $name, $arch, $rver, $dver, $ldver, $tmp);
+	my ($item, $rpm, $name, $arch, $rver, $dver, $ldver, $tmp, $res, $loc);
 	my $BestMatch=1;
 	my $CheckLocal=0;
 
@@ -647,7 +683,12 @@ sub SelectRPMs
 				print "(no local version)\n" if $DEBUG;
 				next;
 			}
-			unless ( $rpm->compare($LocalRPMs{$name}) ==1 ) {
+			$loc= $LocalRPMs{$name};
+			$res= $rpm->compare($loc);
+			if ( $res == -1 or ($res == 0 and ($BestMatch or ($rpm->version ne $loc->version) )) ) {
+				print "(have local version)\n" if $DEBUG;
+				next;
+			} elsif ($res ==0 and IsElement($rpm->arch, @{$LocalRPMsArchs{$name}})) {
 				print "(have local version)\n" if $DEBUG;
 				next;
 			}
@@ -744,8 +785,10 @@ autoupdate.pm
  $rpm->filename
  $rpm1->compare($rpm2)
 
- &SetArch &SetDistArch &GetArch &GetDistArch &GetAllowedArch &GetAllowedDistArch &TestArch
- &SetLocalRPMs &AddLocalRPMs &CheckLocal &SelectRPMs
+ &SetArch &SetDistArch &TestArch
+ &GetArch &GetDistArch &GetAllowedArch &GetAllowedDistArch
+ &SetLocalRPMs &AddLocalRPMs &CheckLocal &CompareLocal &SelectRPMs
+ &SetQueryRpmHeaders &SetQueryRpmDatabase &TieRpmDatabase
  &IsElement
 
 =head1 DESCRIPTION
@@ -755,20 +798,42 @@ manipulate them.
 
 =head2 Header objects
 
-A heaer object can be crated with
+A header object can be crated with
 
  $rpm = Header->new("/path/foo-1.0-1.i386.rpm");
 
-from a file name (the file does not get looked at). Methods implemented are
+from a file name. An optional argument "f", "r", or "l" can
+be used to specify the type.
+
+ "f(ile)" the rpm corresponds to an existing file and can be
+         queried using the rpm bindings.
+ "l(ocal)" the rpm corresponds to a locally installed rpm
+ "r(emote)" the rpm corresponds to a remote file which cannot
+         be queried
+
+The string shouold be of the form "[/path/]name-[epoch:]version-release.arch.rpm".
+
+Methods implemented are
 
  $rpm->name     returns the rpm name (e.g.: foo)
  $rpm->version  returns the rpm version (e.g.: 1.0-1)
  $rpm->arch     returns the rpm architecutre (e.g.: i386)
- $rpm->rpmname  returns the file name (e.g.: foo-1.0-1.i386.rpm)
- $rpm->filename returns the file name (e.g.: /path/foo-1.0-1.i386.rpm)
+ $rpm->rpmname  returns the full rpm name (e.g.: foo-1.0-1.i386.rpm)
+ $rpm->filename returns the full file name (e.g.: /path/foo-1.0-1.i386.rpm)
 
  $rpm1->version($rpm2) compares the versions of $rpm1 and $rpm2.
 
+Each object contains the following date:
+
+ $rpm->{NAME}     name tag of the rpm
+ $rpm->{EPOCH}    epoch tag of the rpm
+ $rpm->{VERSION}  version tag of the rpm
+ $rpm->{RELEASE}  release tag of the rpm
+ $rpm->{ARCH}     arch tag of the rpm
+ $rpm->{FILENAME} the full file name
+ $rpm->{TYPE}     the type (f|l|c)
+ $rpm->{HDR}      pointer to the header object returned by the rpm bindings
+
 =head2 Functions
 
 First of all there the following functions related to architectures defined:
@@ -782,19 +847,31 @@ First of all there the following functions related to architectures defined:
  &GetAllowedDistArch       Returns an array of allowed architectures
  &TestArch                 Checks if the arhitecture is known
 
- &SetLocalRPMs(@rpms)      Takes an array of file names and builds a hash of local rpms
+ &SetLocalRPMs(@rpms)      Takes an array of file names and builds a hash of local                                      rpms
  &AddLocalRPMs(@rpms)      Adds file names to the hash of local rpms
- &CheckLocal($rpm)         Checks if an rpm is in the hash of local rpms and returns its version
+ &CheckLocal($name)        Checks if an rpm is in the hash of local rpms and returns
+                           its version
+ &CompareLocal($rpm)       Compares an rpm against the local version and returns the
+                           result
 
- &SelectRPMs(\@rpms, 0|1, 0|1|2) Selects rpms from the given array by comparing them to the
-                           local hash. Only the rpm with the highest version are selected.
-                           If the second argument is 1, only the rpm is selected for each
-                           version which matches best. If the third argument is 1 only
-                           rpms which are newer than the local one are selected, if it is 2
-                           only rpms which are newer than the local one or for which no
-                           local version exists are selected.
+ &SelectRPMs(\@rpms, 0|1, 0|1|2) Selects rpms from the given array by comparing them
+                           to the local hash. Only the rpm with the highest version
+                           are selected. If the second argument is 1, only the rpm is
+                           selected for each version which matches best. If the third
+                           argument is 1 only rpms which are newer than the local one
+                           are selected, if it is 2 only rpms which are newer than the
+                           local one or for which no local version exists are selected.
 
- &IsElement($item,@array)  Tests if $item is elemnt of @array.
+ &SetQueryRpmHeaders(0|1)  Should rpms (of type file) be queried using the rpm                                          bindings.
+ &SetQueryRpmDatabase(0|1) Should the local rpm data base be queried using the rpm
+                           bindings.
+ &TieRpmDatabase(0|1)      Tie, untie the local rpm data base.
+ 
+ &IsElement($item,@array)  Tests if $item is element of @array.
+
+=head1 SEE ALSO
+
+perl(1), rpm(8), RPM(3)
 
 =head1 AUTHOR
 
